<?php
/**
 * Configuration storage handler classes.
 *
 * Includes following classes user for configuration handling:
 *
 * - class ConfigBase
 *   Base configuration class, other childs must extend this
 *
 * - class ConfigSerializer
 *   Handles configuration data in files
 *
 * - class ConfigDBStorage
 *   Handles configuration data in a table.
 *   NOTE: the table must be created before
 *
 * - class ConfigFactory
 *   Factory class should used to get a instance of a configuration class
 *
 * @author      Murat Purc <murat@purc.de>
 * @copyright    Murat Purc 2008
 * @date        18.04.2008
 * @package     Contenido
 * @subpackage  Configuration
 * @todo:       Error handling
 */


/**
 * Base config class. Extended classes must overwrite existing Methods.
 * 
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  Configuration
 */
class ConfigBase {

    /**
     * Flag about occured error
     * @var  bool
     * @access  protected
     */
    var $_bError = false;
    
    /**
     * Key of configuration (could be a file or a md5 hash, depends on child class)
     * @var  string
     * @access  protected
     */
    var $_key = null;
    
    
    /**
     * Constructor of ConfigBase, does some checks, stops further script execution if validation of
     * arguments fails.
     * 
     * @param  array  $options  Assoziative options array as follows:
     * <code>
     * $options['key'] = Configuration key, depending on child class
     * </code>
     */
    function ConfigBase($options) {
        if (!is_array($options)) {
            die('ConfigBase->ConfigBase: Invalid argument');
        }
        
        if (!isset($options['key'])) {
            die('ConfigBase->ConfigBase: Missing argument $options["key"]');
        }
        
        $this->_key = $options['key'];
    }

    /**
     * Method to get configuration, must be overwritten by child.
     * 
     * A direct call of this method will result in termination of script execution.
     */
    function get(){
        die("ConfigBase->get: Method must be overwritten by child!");
    }

    /**
     * Method to set configuration, must be overwritten by child.
     * 
     * A direct call of this method will result in termination of script execution.
     * 
     * @param  mixed  $param  Assoziative array or another type of variable
     */
    function set($param){
        die("ConfigBase->set: Method must be overwritten by child!");
    }

    /**
     * Method to remove configuration, must be overwritten by child.
     * 
     * A direct call of this method will result in termination of script execution.
     */
    function remove(){
        die("ConfigBase->remove: Method must be overwritten by child!");
    }

}


/**
 * ConfigSerializer to store serialized configuration in files.
 * 
 * Usage:
 * <code>
 * $options['key'] = '/full/path/to/config.php';
 * $config = ConfigFactory::get('serializer', $options);
 * </code>
 *
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  Configuration
 */
class ConfigSerializer extends ConfigBase {

    /**
     * Constructor of ConfigSerializer, delegates parameter to parent.
     * 
     * @param  array  $options  Assoziative options array as follows:
     * <code>
     * $options['key'] = Full path to configuration file
     * </code>
     */
    function ConfigSerializer($options) {
        parent::ConfigBase($options);
    }

    
    /**
     * Reads content of configuration file and returns the content as a unserializen PHP variable.
     * 
     * @return  mixed  PHP variable or nul on error
     */
    function get(){
        if (!is_file($this->_key)) {
            return null;
        }
        if ($content = file_get_contents($this->_key)) {
            return unserialize($content);
        } else {
            return null;
        }
    }

    
    /**
     * Serializes passed configuration and stores it into the configuation file
     * 
     * @return  bool  True on success otherwhise false
     */
    function set($content){
        // use LOCK_EX if script is running under PHP5
        $flag = (defined('LOCK_EX')) ? LOCK_EX : null;
        $result = file_put_contents($this->_key, serialize($content), $flag);
        return ($result) ? true : false;
    }

    
    /**
     * Removes (unlink) configuration file.
     * 
     * @return  bool  True on success otherwhise false
     */
    function remove(){
        if (is_file($this->_key)) {
            return unlink($this->_key);
        }
        return false;
    }
    
}


/**
 * Class ConfigDBStorage to store serialized configuration in a table.
 *
 * Needs a table, create it if not exists using following statement:
 * <code>
 * -- {prefix} ist almost con by default
 * CREATE TABLE {prefix}_configdbstorage (
 *     `id` varchar(32) NOT NULL default '' COMMENT 'md5 hash as PK',
 *     `value` text COMMENT 'Serialized content of configuration',
 *     PRIMARY KEY (`id`)
 * )
 * </code>
 * 
 * Usage:
 * <code>
 * // md5 hash of $options['key'] will be used as id in table {prefix}_configdbstorage
 * $options['key'] = '/full/path/to/config.php';
 * $config = ConfigFactory::get('dbstorage', $options);
 * </code>
 * 
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  Configuration
 */
class ConfigDBStorage extends ConfigBase {

    /**
     * Database instance
     * @var  DB_Contenido
     */
    var $_oDB = null;
    
    /**
     * Name of table where the configuration is stored
     * @var  string
     */
    var $_table = '';

    
    /**
     * Constructor of ConfigDBStorage, delegates parameter to parent and sets some properties
     * 
     * @param  array  $options  Assoziative options array as follows:
     * <code>
     * $options['key'] = Normally full path to configuration file, where the md5 hash will be calculated
     * </code>
     */
    function ConfigDBStorage($options) {
        parent::ConfigBase($options);
        $this->_key   = md5($this->_key);
        $this->_table = $GLOBALS['cfg']['sql']['sqlprefix'] . '_configdbstorage';
        $this->_oDB   = new DB_Contenido();
    }

    
    /**
     * Returns the configuration from db table
     *
     * @return  mixed  PHP variable or nul on error
     */
    function get(){
        $sql = 'SELECT value FROM ' . $this->_table . ' WHERE id="' . $this->_key . '"';
        $this->_oDB->query($sql);
        if ($this->_oDB->next_record()) {
            return unserialize($this->_oDB->f('value'));
        } else {
            return null;
        }
    }

    
    /**
     * Serializes and writes passed configuration into the table.
     * 
     * @return  bool  True on success otherwhise false
     */
    function set($content){
        $content = mysql_real_escape_string(serialize($content));
        if ($this->get()) {
            $sql = 'UPDATE ' . $this->_table . ' SET value="' . $content . '" WHERE id="' . $this->_key . '"';
        } else {
            $sql = 'INSERT INTO ' . $this->_table . ' (`id`, `value`) VALUES ("' . $this->_key . '", "' . $content . '")';
        }
        $result = $this->_oDB->query($sql);
        return ($result) ? true : false;

    }

    
    /**
     * Removes the configuration from table.
     * 
     * @return  bool  True on success otherwhise false
     */
    function remove(){
        $sql = 'DELETE FROM ' . $this->_table . ' WHERE id="' . $this->_key . '"';
        $result = $this->_oDB->query($sql);
        return ($result) ? true : false;
    }
    
}



/**
 * Factory class to get an instance of one of configuration classes
 *
 * <code>
 * // ConfigSerializer example
 * $options['key'] = '/full/path/to/config.php';
 * $config = ConfigFactory::get('serializer', $options);
 *
 * // ConfigDBStorage example
 * $options['key'] = '/full/path/to/config.php';
 * $config = ConfigFactory::get('dbstorage', $options);
 * </code>
 * 
 * @author      Murat Purc <murat@purc.de>
 * @package     Contenido
 * @subpackage  Configuration
 */
class ConfigFactory {

    /**
     * Returns a instance of desired configuration class
     * 
     * @param   string  $name  Type of configuration ('serializer' or 'dbstorage')
     * @param   array   $options  Options array
     * @return  ConfigBase  A child of of ConfigBase class
     */
    function get($name, $options) {
        if ($name == 'serializer') {
            return new ConfigSerializer($options);
        } elseif ($name == 'dbstorage') {
            return new ConfigDBStorage($options);
        }
        return new ConfigSerializer($options);
        
    }

}


/**
 * Emulate file_put_contents fpr PHP 4
 */
if (!function_exists('file_put_contents') && !defined('FILE_APPEND')) {

    /**#@+
     * Constants
     */
    /**
     * File append option for file_put_contents()
     */
    define('FILE_APPEND', 1);
    
    /**
     * This function is identical to calling fopen(), fwrite() and fclose() successively to write data to a file. 
     *
     * @param  string  $filename  Path to the file where to write the data
     * @param  mixed   $data      The data to write. Can be either a string or a array
     * @param  int     $flag      Flag, supported is only FILE_APPEND (=1)
     * @return mixed   Number of written bytes or false on error
     */
    function file_put_contents($filename, $data, $flag=false) {
        $mode = ($flag == FILE_APPEND || strtoupper($flag) == 'FILE_APPEND') ? 'a' : 'w';
        if (!$f = @fopen($filename, $mode)) {
            return false;
        }
        if (is_array($data)) {
            $data = implode($data);
        }
        $bytes_written = fwrite($f, $data);
        fclose($f);
        return $bytes_written;
    }

}
